package connect.demo;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.util.Properties;

import org.apache.commons.codec.binary.Base64;
import org.bouncycastle.util.io.pem.PemObject;
import org.bouncycastle.util.io.pem.PemReader;

/**
 * 
 * @author SAASPASS Developers
 *
 */
public class Util {

	public static Properties getConfigProperties() throws IOException {
		Properties prop = new Properties();
		InputStream inputStream = null;
		try {
			String propFileName = "/conf.properties";
 
			inputStream = Util.class.getClassLoader().getResourceAsStream(propFileName);
 
			if (inputStream != null) {
				prop.load(inputStream);
			} else {
				throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
			}
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			inputStream.close();
		}
		return prop;
	}
	
	public static String base64UrlDecode(String input) {
        byte[] decodedBytes = base64UrlDecodeToBytes(input);
        String result = new String(decodedBytes, StandardCharsets.UTF_8);
        return result;
    }

    public static byte[] base64UrlDecodeToBytes(String input) {
        Base64 decoder = new Base64();//(-1, null, true);
        byte[] decodedBytes = decoder.decode(input.getBytes());
        return decodedBytes;
    }

    /**
     * Read public key from a file in PEM format. You may give key in String format instead of reading from file.
     * @return
     * @throws FileNotFoundException
     * @throws IOException
     * @throws NoSuchAlgorithmException
     * @throws NoSuchProviderException
     */
    public static PublicKey readPublicKey() throws FileNotFoundException, IOException, NoSuchAlgorithmException, NoSuchProviderException {
    	KeyFactory factory = KeyFactory.getInstance("RSA", "BC");
		try {
			PublicKey pub = readPublicKey(factory, "/pk.pub");
			return pub;
		} catch (InvalidKeySpecException e) {
			e.printStackTrace();
			System.out.println("invalid key spec");
		}
		return null;
    }
    
    /**
     * Read public key content from file.
     * @param factory
     * @param filename
     * @return
     * @throws InvalidKeySpecException
     * @throws FileNotFoundException
     * @throws IOException
     */
    public static PublicKey readPublicKey(KeyFactory factory, String filename) throws InvalidKeySpecException, FileNotFoundException, IOException {
    	InputStream in = Util.class.getClassLoader().getResourceAsStream(filename);
    	PemReader pemReader = new PemReader(new InputStreamReader(in));
    	byte[] content = null;
		try {
			PemObject pemObject = pemReader.readPemObject();
			content = pemObject.getContent();
		} finally {
			pemReader.close();
		}
		X509EncodedKeySpec pubKeySpec = new X509EncodedKeySpec(content);
		return factory.generatePublic(pubKeySpec);
	}
	
}
